@@ -1,4 +1,4 @@
-/* MP4H -- A macro processor for HTML documents
+/* mp4h -- A macro processor for HTML documents
    Copyright 2000, Denis Barbier
    All rights reserved.
 
@@ -124,17 +124,17 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
 
 
 /*-------------------------------------------------------------------------.
-| This function parses one argument to a macro call.  It expects the first |
-| left parenthesis, or the separating comma to have been read by the       |
-| caller.  It skips leading whitespace, and reads and expands tokens,      |
-| until it finds a comma or an right parenthesis at the same level of      |
-| parentheses.  It returns a flag indicating whether the argument read are |
-| the last for the active macro call.  The argument are build on the       |
-| obstack OBS, indirectly through expand_token ().                         |
+| This function parses one argument to a macro call.  It skips leading     |
+| whitespace and reads and expands tokens until it finds a space outside   |
+| of any group, or a right angle bracket.  It returns a flag indicating    |
+| whether the argument read are the last for the active macro call.  The   |
+| argument read are the last for the active macro call.  The argument are  |
+| on the obstack OBS, indirectly through expand_token ().                  |
 `-------------------------------------------------------------------------*/
 
 static boolean
-expand_argument (struct obstack *obs, read_type expansion, token_data *argp)
+expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
+                 char *last_char_ptr)
 {
   char *text;
   token_type t;
@@ -142,6 +142,7 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp)
   int group_level = 0;
   boolean in_string = FALSE;
 
+  *last_char_ptr = ' ';
   TOKEN_DATA_TYPE (argp) = TOKEN_VOID;
 
   /* Skip leading white space.  */
@@ -182,29 +183,32 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp)
           break;
 
         case TOKEN_BGROUP:
-          group_level++;
-          if (expansion == READ_ATTR_BODY)
+          if (expansion == READ_ATTR_ASIS && group_level == 0)
             obstack_1grow (obs, CHAR_BGROUP);
+          group_level++;
           break;
 
         case TOKEN_EGROUP:
-          if (expansion == READ_ATTR_BODY)
-            obstack_1grow (obs, CHAR_EGROUP);
           group_level--;
+          if (expansion == READ_ATTR_ASIS && group_level == 0)
+            obstack_1grow (obs, CHAR_EGROUP);
           break;
 
         case TOKEN_QUOTE:
           if (group_level == 0)
             in_string = !in_string;
+          if (expansion_level > 1)
+            obstack_grow (obs, TOKEN_DATA_TEXT (&td),
+                    strlen (TOKEN_DATA_TEXT (&td)));
           break;
 
         case TOKEN_QUOTED:
         case TOKEN_STRING:
-          if (expansion_level > 0 && t == TOKEN_QUOTED)
+          if (t == TOKEN_QUOTED)
             obstack_1grow (obs, CHAR_LQUOTE);
           obstack_grow (obs, TOKEN_DATA_TEXT (&td),
                   strlen (TOKEN_DATA_TEXT (&td)));
-          if (expansion_level > 0 && t == TOKEN_QUOTED)
+          if (t == TOKEN_QUOTED)
             obstack_1grow (obs, CHAR_RQUOTE);
           break;
 
@@ -227,6 +231,7 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp)
           abort ();
         }
 
+      *last_char_ptr = *(TOKEN_DATA_TEXT (&td) + strlen (TOKEN_DATA_TEXT (&td)) - 1);
       t = next_token (&td, expansion);
     }
 }
@@ -235,9 +240,11 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp)
 | Collect all the arguments to a call of the macro SYM.  The arguments are |
 | stored on the obstack ARGUMENTS and a table of pointers to the arguments |
 | on the obstack ARGPTR.                                                   |
+| If there is a slash character before closing bracket, this function      |
+| returns TRUE, otherwise FALSE.                                           |
 `-------------------------------------------------------------------------*/
 
-static void
+static boolean
 collect_arguments (symbol *sym, read_type expansion, struct obstack *argptr,
                    struct obstack *arguments)
 {
@@ -246,6 +253,7 @@ collect_arguments (symbol *sym, read_type expansion, struct obstack *argptr,
   token_data *tdp;
   char *last_addr;
   boolean more_args;
+  char last_char;
 
   TOKEN_DATA_TYPE (&td) = TOKEN_TEXT;
   TOKEN_DATA_TEXT (&td) = SYMBOL_NAME (sym);
@@ -262,7 +270,7 @@ collect_arguments (symbol *sym, read_type expansion, struct obstack *argptr,
           /*  remember last address in use to remove the last
               argument if it is empty.  */
           last_addr = argptr->next_free;
-          more_args = expand_argument (arguments, expansion, &td);
+          more_args = expand_argument (arguments, expansion, &td, &last_char);
           if (internal_abort)
             {
               MP4HERROR ((EXIT_FAILURE, 0,
@@ -281,10 +289,13 @@ collect_arguments (symbol *sym, read_type expansion, struct obstack *argptr,
           && strlen (TOKEN_DATA_TEXT (tdp)) == 0)
         argptr->next_free = last_addr;
     }
+  return (last_char == '/');
 }
 
 /*-----------------------------------------------------------------.
-| Collect the body of a container tag. No expansion is performed.  |
+| Collect the body of a container tag. No expansion is performed,  |
+| but when a macro is found its arguments are collected.  This is  |
+| necessary to deal with nested expressions.                       |
 `-----------------------------------------------------------------*/
 static void
 collect_body (symbol *sym, struct obstack *argptr, struct obstack *bodyptr)
@@ -372,7 +383,7 @@ collect_body (symbol *sym, struct obstack *argptr, struct obstack *bodyptr)
               if (newsym == NULL || SYMBOL_TYPE (newsym) == TOKEN_VOID)
                 shipout_text (bodyptr, TOKEN_DATA_TEXT (&td), strlen (TOKEN_DATA_TEXT (&td)));
               else
-                expand_macro (newsym, READ_ATTR_BODY);
+                expand_macro (newsym, READ_ATTR_ASIS);
             }
           break;
 
@@ -434,12 +445,13 @@ call_macro (symbol *sym, struct obstack *obs, int argc, token_data **argv,
 static void
 expand_macro (symbol *sym, read_type expansion)
 {
-  struct obstack arguments, argptr, save_argptr, body;
+  struct obstack arguments, argptr, body;
   token_data **argv;
   int argc, i;
   struct obstack *obs_expansion;
   const char *expanded;
-  boolean traced;
+  char *cp;
+  boolean traced, slash;
   int my_call_id;
   read_type attr_expansion;
 
@@ -462,22 +474,33 @@ expand_macro (symbol *sym, read_type expansion)
   if (traced && (debug_level & DEBUG_TRACE_CALL))
     trace_prepre (SYMBOL_NAME (sym), my_call_id);
 
-  if (expansion == READ_ATTR_BODY || expansion == READ_BODY)
-    attr_expansion = READ_ATTR_BODY;
-  else if (expansion == READ_ATTR_VERB || ! SYMBOL_EXPAND_ARGS (sym))
+  if (expansion == READ_ATTR_ASIS || expansion == READ_BODY
+          || expansion == READ_ATTR_VERB)
+    attr_expansion = READ_ATTR_ASIS;
+  else if (! SYMBOL_EXPAND_ARGS (sym))
     attr_expansion = READ_ATTR_VERB;
   else
     attr_expansion = READ_ATTRIBUTE;
 
-  collect_arguments (sym, attr_expansion, &argptr, &arguments);
-  save_argptr = argptr;
+  slash = collect_arguments (sym, attr_expansion, &argptr, &arguments);
   argc = obstack_object_size (&argptr) / sizeof (token_data *);
-  argv = (token_data **) obstack_finish (&argptr);
 
   if (SYMBOL_CONTAINER (sym))
     {
-      collect_body (sym, &save_argptr, &body);
-      argv = (token_data **) obstack_finish (&save_argptr);
+      collect_body (sym, &argptr, &body);
+      argv = (token_data **) obstack_finish (&argptr);
+    }
+  else
+    {
+      argv = (token_data **) obstack_finish (&argptr);
+      if (slash)
+        {
+          cp = TOKEN_DATA_TEXT (argv[argc-1]);
+          if (*cp == '/' && *(cp+1) == '\0')
+            argc--;
+          else if (*(cp + strlen (cp) - 1) == '/')
+            *(cp + strlen (cp) - 1) = '\0';
+        }
     }
 
   if (traced)
@@ -500,19 +523,24 @@ expand_macro (symbol *sym, read_type expansion)
       for (i = 1; i < argc; i++)
         {
           obstack_1grow (obs_expansion, ' ');
-          if (strlen (TOKEN_DATA_TEXT (argv[i])) == 0)
-            {
-              obstack_1grow (obs_expansion, CHAR_BGROUP);
-              obstack_1grow (obs_expansion, CHAR_EGROUP);
-            }
-          else if (strchr (TOKEN_DATA_TEXT (argv[i]), ' '))
-            {
-              obstack_1grow (obs_expansion, CHAR_BGROUP);
-              shipout_string (obs_expansion, TOKEN_DATA_TEXT (argv[i]), 0);
-              obstack_1grow (obs_expansion, CHAR_EGROUP);
-            }
-          else
+          if (expansion == READ_ATTR_ASIS)
             shipout_string (obs_expansion, TOKEN_DATA_TEXT (argv[i]), 0);
+          else
+            {
+              if (strlen (TOKEN_DATA_TEXT (argv[i])) == 0)
+                {
+                  obstack_1grow (obs_expansion, CHAR_BGROUP);
+                  obstack_1grow (obs_expansion, CHAR_EGROUP);
+                }
+              else if (strchr (TOKEN_DATA_TEXT (argv[i]), ' '))
+                {
+                  obstack_1grow (obs_expansion, CHAR_BGROUP);
+                  shipout_string (obs_expansion, TOKEN_DATA_TEXT (argv[i]), 0);
+                  obstack_1grow (obs_expansion, CHAR_EGROUP);
+                }
+              else
+                shipout_string (obs_expansion, TOKEN_DATA_TEXT (argv[i]), 0);
+            }
         }
       obstack_1grow (obs_expansion, '>');
       if (SYMBOL_CONTAINER (sym))
